#ifndef CCIRCUIT_HH
#define CCIRCUIT_HH

#include <vector>
#include <xml/Parser.hh>
#include <electro/CElement.hh>




namespace RLC{
class CNetworksArea;
}

/**
 *  Klasa przechowujaca obwod.
 *  To _jest_ full wypas klasa ktora bedzie robila cala czarna robote numeryczna
 *  i posredniczyla miegdzy gui a silnikiem. 
 * /ten opis pozostal ze wzgledow historycznych, prosze wybaczyc jego forme :)/
*/
class CCircuit {
  friend class CElement;
  friend class CVoltage;
  friend class CCurent;
  friend class CResistor;
  friend class CCapasistor;
  friend class CInductor;
  friend class CCurent_u;
  friend class CCurent_i;
  friend class CVoltage_u;
  friend class CVoltage_i;
  friend class CNode;

public:	
      /**
       * Konstruktor.
       * @see ~CCircuit();
       */
       CCircuit() { m_ParentDrawingArea = 0; }
      /**
       * Destructor.
       * @see CCircuit();
       */        
       ~CCircuit();
       /**
        * Dodaje element do obwodu.
        * @param *x wskaznik do elementu
        */ 
	std::vector<CElement *>::iterator items_pos;
       /**
        * Dodaje element do obwodu.
        * @param *x wskaznik do elementu
        */ 
	void add_item(CElement * x){items.push_back(x);}
       /**
        * Laczy 2 elementy (tworzy wirtualne polaczenie miedzy odpowiednimi pinami).
        * @param nameA nazwa pierwszego elementu do polaczenia
        * @param pinA numer pinu pierwszego elementu
        * @param nameB nazwa drugiego elementu do polaczenia
        * @param pinB numer pinu drugiego elementu
        */
        void conect_items(std::string nameA,unsigned int pinA,std::string nameB,unsigned int pinB);
       /**
        * Wypisuje wszyskie elementy , stworzone w celu testowym.
        */
        void print_items();
       /**
        * Wczytuje obwod  z pliku.
        * @param we nazwa pliku wejciowego
        */
	void load(const std::string & we);
       /**
        * Czysci obwod.
        */
        void clear();
       /**
	* Funkcja rysujaca obwod.
	*/ 
	void draw();
       /** 
	* Metoda uzywana do ustawienia na wskaznika m_parent 
	* na obiekt CNetworksArea, w ktorym obwod bedzie
	* rysowany.
	* @see m_ParentDrawingArea
	*/ 
        void setParent(RLC::CNetworksArea* parent);
       /** Wektor wskaznikow do elementow. */ 
        std::vector<CElement *> items;
protected:
       /** Wyrzuca wyjatek */ 
        void throwError(const string& inMessage) const;
       /** Zwraca liczbe elementow w obwodzie */ 
        int count_items(){return items.size();}
       /** Zwraca wskaznik do elementu o zadanej nazwie */ 
        CElement * get_item(std::string s);
       /** 
	* Wskaznik do CNetworksArea, gdzie bedzie rysowany obwod.
	* @se setParent
	*/
	RLC::CNetworksArea* m_ParentDrawingArea;
private:       
       /** Nazwa pliko z ktorego zostal wczytany obwod. */ 
        std::string filein;        
};

#endif // CCIRCUIT_HH
