#include <electro/CResistor.hh>
#include <electro/CVoltage.hh>
#include <electro/CCurent.hh>
#include <electro/CCurent_u.hh>

#include <CCircuitSolver.hh>
#include <cmath>

/**************************************************************************************

  Class Functions (construcotrs & destrucotrs,printers, controls)
 
**************************************************************************************/

CCircuitSolver::~CCircuitSolver() {
    for(int i=0;i<branches.size();i++)
        delete branches[i];
    for(int i=0;i<exnodes.size();i++)
        delete exnodes[i]; 
}

void CCircuitSolver::solve()
{
 cout << "Making new topology\n";
 try{
   zciagaj();
 }
 catch(...){
   cerr<<"Blad podczas zciagania !";
   exit(1);
 }
 
 try{
   make_branches();
 }
 catch(...){
   cerr<<"Blad podczas tworzenia galezi !";
   exit(1);
 }
 
 
 print_aliases(); 
 found_omegas();
 cout << "Omegas in Circuit : ";
 print_omegas();
 
 for(int j=0;j<m_omegas.size();++j)
 {
   double omega=m_omegas[j];
   cout<< "Solving Circuit for omega="<<omega<<"\n";
//   cout << "*** Wyniki ***"<<endl;
   makeA();
   makeE(omega);
   makeI(omega);
   makeYb(omega);
   makeYn(omega);
   makeIn();
   makeUn();
   
   if(false)
   {
   cout << "Macierz incydencji A:"<<endl;
   A->print();
   cout << "Wymuszenia napiciowe stae E:"<<endl;
   E->print();
   cout << "Wymuszenia prdowe stae I:"<<endl;
   I->print();
   cout << "Macierz admitancji gaziowych Yb:"<<endl;
   Yb->print();
   cout << "Macierz admintacji wzowych Yn:"<<endl;
   Yn->print();
   cout << "Wektor wydajnoci prdowych In:"<<endl;
   In->print();
   Un->print();
   }
   
   //tutaj mamy juz pieknie wszysko rozwiazane 
   //wypisujemy kontrolnie
   cout << "\tVoltages:\n\t";
   for(unsigned int i=0;i<exnodes.size()-1;i++)
    cout<<"["<<i<<"]="<<(*Un)[0][i]<<"\t";  
   
   //teraz wtawiamy odpowiednie fale do naszych sygnalow 
   for(unsigned int i=0;i<exnodes.size()-1;i++)
     exnodes[i]->signal->addWave((*Un)[0][i].getM(),omega,(*Un)[0][i].getFi());
   
   // do naszego weza odniesienia wtawiamy zero
   exnodes[exnodes.size()-1]->signal->addWave(0,0,0);

   //teraz to co gumisie lubia najbardziej
   //czyli musimy wyznaczyc prady
   for(unsigned int i=0;i<branches.size();i++){
     branches[i]->print_branch();
     //to jest troszke niebezpieczne ... ale nie przewiduje zeby moglo spowodowac wykrzaczenie
     int start=(dynamic_cast<CExNode *>(branches[i]->elements[0]))->nr;
     int koniec=(dynamic_cast<CExNode *>(branches[i]->elements[branches[i]->elements.size()-1]))->nr;
     if(branches[i]->getI(omega)!=Complex(0,0))
       branches[i]->signal->addWave(branches[i]->getI(omega).getM(),omega,branches[i]->getI(omega).getFi());
     else if(branches[i]->getZ(omega)!=Complex(0,0))
       {
       cout<<"U/R--"<<start<<"-"<<koniec<<endl;
       Complex u=(*Un)[0][koniec]-(*Un)[0][start];
       Complex z=branches[i]->getZ(omega);
       Complex j=u/z;
       branches[i]->signal->addWave(j.getM(),omega,j.getFi());
       }
     else ;
      //cout<<"Err"<<endl;
        
       
     
     }
//     exnodes[i]->signal->addWave((*Un)[0][i].getM(),omega,(*Un)[0][i].getFi());
      
  
  cout<<"\n";
  }
  
  for(int i=0;i<m_data->items.size();i++)
  {
    CElement * p = m_data->items[i];
    if(p && p->signal)  
        cout << p->getName()<<"\t"<<p->signal->getEq()<<" ["<<p->signal->getUnit()<<"]\n";
    
  } 
  
}

void CCircuitSolver::print_omegas(){
  for(int i=0;i<m_omegas.size()-1;++i)
     cout<<m_omegas[i]<<", ";
  cout<<m_omegas[m_omegas.size()-1]<<"\n";
}

void CCircuitSolver::found_omegas(){
 CElementActive *p;
 //przelec po wszyskich elementach ukladu
 for(int i=0;i<m_data->items.size();++i)      
   //jesli nasz element jest aktywny, tzn daje wlasny wklad do ukladu to 
   if(p=dynamic_cast<CElementActive *> (m_data->items[i]))
    { 
     //przelec po wszyskich omegach i sprawdz czy jeszcze taka nie byla dodana
     bool founded=false;
     for(int j=0;j<m_omegas.size();++j)
       //jesli znajdziemy juz taka to wyskakujemy z petli
       if(p->getOmega()==m_omegas[j]) 
         {
            founded=true;
            break;
         }   
     //jesli nie znalezlismy to dodajemy
     //cout << "|"<<p->getOmega()<<"| ";
     if(!founded)m_omegas.push_back(p->getOmega());
    } 
}

void CCircuitSolver::print_aliases()
{

cout << "Real nodes in circuit ("<<exnodes.size()<< ") :\n";
for(int i=0;i<exnodes.size();i++)
  exnodes[i]->print_conections();


cout << "Branches ("<<branches.size()<<") :\n";
for(int i=0;i<branches.size();i++)
    {
    cout << "["<<i<< "] :: ";
     branches[i]->print_branch();
    } 
  
}

void CCircuitSolver::clear(){
//teraz musimy wywalic wszyko co sie tylko da :]

//zaczynamy od sygnalow 
try{
/*
  for(int i=0;i<exnodes.size();i++){
    if(exnodes[i]->signal) delete exnodes[i]->signal;
    exnodes[i]->signal=0;
    }
  for(int i=0;i<branches.size();i++){
    if(branches[i]->signal) delete branches[i]->signal;
    branches[i]->signal=0;
    }
*/
}
catch(...){
  throw CRlcException("CCircuitSolver::clean() : Nie oczekiwany blad podczas czyszczenia sygnalow.");
}
//usun obiekty wierzcholkow i galezi
try{
/*
  for(int i=0;i<branches.size();i++)
    if(branches[i]) delete branches[i];
  for(int i=0;i<exnodes.size();i++)
    if(exnodes[i]) delete exnodes[i];
*/
}
catch(...){
 throw CRlcException("CCircuitSolver::clean() : Nie oczekiwany blad podczas czyszczenia elementow pomocniczych.");
}
//wyczyc listy wierzcholkow i galezci
exnodes.clear();
branches.clear();
//wyzeruj wskaznik do Obwodu wejsciowego
m_data=0;

};


/**************************************************************************************

  Graf Functions (making grafs, branches, exnodes)
 
**************************************************************************************/

CElement * CCircuitSolver::get_item(std::string s)
{
   for(int i=0;i<m_data->items.size();++i)      
      if((*m_data->items[i]).getName()==s) return m_data->items[i];
   return NULL;
}

void CCircuitSolver::serch_nodes(CExNode *des, CNode *start) {
if(start)//jesli wskaznik jest nie zerowy
{
   CNode * p = dynamic_cast<CNode *>(start); // to probujemy rzutowac
   if(p){                                     // jesli mamy wierzcholek to super :)
      for(int i=0;i<4;i++){                   // probujemy z tego wierzcholka wybiegac do przodu przez 4 piny
         CPin *pin = p->pins[i];
         if(pin->element){                     // jesli pin jest != (tzn ze do czegos prowadzi)
            CNode * x= dynamic_cast<CNode *>(pin->element);  // sprawdzamy do czego
            if(x){                             //jesli do kolejnego wierzcholka
               if(!used.was_used(x)){          //jesli nie byl smigany
                  used.use(x);
                  x->papa=des;
                  x->signal=des->signal;
                  serch_nodes(des,x);          //to tam idziemu
               }
            }
            else                               //jesli pin jest elementem
               des->add(pin);                  //to go wciagamy na liste
         }
       }
      }
   }
};

void CCircuitSolver::zciagaj()
{

  for(int i=0;i<m_data->items.size();i++)
  {
    CNode * p = dynamic_cast<CNode *>(m_data->items[i]);
    if(p)  // jesli znalezlismy obiekt typu node
      if(! used.was_used(p))
        {       
          exnodes.push_back(new CExNode(p->getName()));
          used.use(p);
          
          exnodes[exnodes.size()-1]->nr=exnodes.size()-1;
          exnodes[exnodes.size()-1]->signal=new CSignal("voltage","V");
          p->signal=exnodes[exnodes.size()-1]->signal;  
          p->papa=exnodes[exnodes.size()-1];
          serch_nodes(exnodes[exnodes.size()-1],p); // brzydko !! 
        }
  }
  
}

void CCircuitSolver::make_branches(){

int no=0;

  for(unsigned int i=0;i<exnodes.size();i++)
    {
       for(int j=0;j<exnodes[i]->count_pins();j++)
           {          
               CBranch *b=new CBranch ;
               b->signal = new CSignal("Prad","A");
//               cout << "\tScierzka "<<i<<endl;
               branches.push_back(b);
               b->nr=no;
               no++;
               (b->elements).push_back(exnodes[i]);
               CElement *p=(*(exnodes[i]->pins[j])).element;
               int q=(*(exnodes[i]->pins[j])).pin;
               //trzeba jeszcze zrobic jak klika pod rzad bedzie mialo wierzcholki na poczatku
               while(!dynamic_cast<CNode *> (p))
                 { 
                      //cout<< "\t\t jestem w "<<p->getName()<<"."<<q<< endl;  
                        if(p->branch) // jesli nasz element nalerzy juz do jakiejs galazki to 
                        {
                          
                          //cout << "\tzoonk "<<j<<endl;
                          no--; //dekrementuj liczbe galezi
                          branches.pop_back();
                          delete b;
                          p=NULL;
                          break;
                        }
                        
                        //dajemy elementowi wskaznik do galezi
                        p->branch=b;
                        //podpinamy sygnaly
                        p->signal=b->signal;
                        signed int sign=1;
                        if(q==0)
                           sign=-1;
                        else if(q==1)   
                           sign=1;
                        
                        //if(dynamic_cast<CResistor *> (p)) 
   //                             b->Z = b->Z+(dynamic_cast<CResistor *>(p))->getR();
                        //else if(dynamic_cast<CVoltage *> (p)) 
                               //b->U = b->U+sign*(dynamic_cast<CVoltage *>(p))->getU();
                        //else if(dynamic_cast<CCurent *>
                              //(p)) b->I = b->I+sign*(dynamic_cast<CCurent *>(p))->getI();
                              
                        q++;
                        q%=2;
                        int q_tmp=(*p->pins[q]).pin;
                        (b->elements).push_back(p);
                        p=(*p->pins[q]).element;
                        //cout << "\t\t ide do "<<p->getName()<<"."<<q<<endl;
                        q=q_tmp;
                        
                 }
                 if(p)
                   {
                    p=(dynamic_cast<CNode *> (p))->papa; 
                    (b->elements).push_back(p);
                   } 
                
                   
               
           }    
    }// tutaj juz mamy utworzone wszelkie galezie
    
}


/**************************************************************************************

  Matrix Functions (making topology matrix)
 
**************************************************************************************/

void CCircuitSolver::makeA()
{
  A=new CMatrix<Complex>(exnodes.size(),branches.size());
  
  for(int i=0;i<branches.size();i++)
  {
    int start=(*(dynamic_cast<CExNode *> (branches[i]->elements[0])) ).nr;
    int koniec=(*(dynamic_cast<CExNode *> (branches[i]->elements[branches[i]->elements.size()-1])) ).nr;
     
    (*A)[start][i]=1;
    (*A)[koniec][i]=-1;
  }
  A->setRows(exnodes.size()-1);
}

void CCircuitSolver::makeYb(double omega)
{
  Yb=new CMatrix<Complex>(branches.size(),branches.size());
  Yb->zero();
  for(int i=0;i<branches.size();i++)
  {
    //jesli galaz ma jakas impedencja to ja wstawiamy
    if(branches[i]->getZ(omega)!=Complex(0,0))
      (*Yb)[i][i]=(Complex)1/branches[i]->getZ(omega);
    
    // pozatym szukamy czy nie ma jakisegos zrodelka sterowanego :)
    for(int j=0;j<branches[i]->elements.size();j++)
     {
      CCurent_u * p = dynamic_cast<CCurent_u *>(branches[i]->elements[j]); // to probujemy rzutowac
      if(p)      
         {
           CElement *x=get_item(p->getOn());
           // cout << "trabka ! :D ["<<i<<","<<j<<"] "<<x->branch->nr <<endl;
           (*Yb)[i][x->branch->nr]=(Complex)p->getGm();
           }
     }
    
  }

        
}


void CCircuitSolver::makeYn(double omega)
{
  Yn=new CMatrix<Complex>(exnodes.size(),1);
  (*Yn)=(*A)*(*Yb)*(A->T());
}

void CCircuitSolver::makeIn()
{
  In=new CMatrix<Complex>(exnodes.size(),1);
  (*In)=(*A)*( (*I) - (*Yb)*(*E));
}

void CCircuitSolver::makeUn()
{
  Un=new CMatrix<Complex>(exnodes.size(),1);
  Yn->invert();
  (*Un)=(*Yn)*(*In);
}

void CCircuitSolver::makeE(double omega)
{
  E=new CMatrix<Complex>(branches.size(),1);
  for(int i=0;i<branches.size();i++)
    (*E)[i][0]=branches[i]->getU(omega);
}

void CCircuitSolver::makeI(double omega)
{
  I=new CMatrix<Complex>(branches.size(),1);
  for(int i=0;i<branches.size();i++)
    (*I)[i][0]=branches[i]->getI(omega);
}



/*
class CExNode : public CElement
{
protected: 

public:
    CExNode(std::string name,int x=0,int y=0) : CElement("NODE",name,x,y){};
    ~CExNode(){
         for(int i=0;i<pins.size();i++)
            delete pins[i];
    }
    void add(CPin * item){
       pins.push_back(item);
    }
    int count_pins(){return pins.size();}

};

*/


