#ifndef CCIRCUITSOLVER_HH
#define CCIRCUITSOLVER_HH

#include <math/Complex.hh>
#include <math/CMatrix.hh>
#include <electro/CElement.hh>
#include <CCircuit.hh>
#include <CRlcException.hh>


/**
 * Klasa przetrzymuje informacje podczas zciaganie wierzcholkow
 */
  class CUsedList
  { 
    public : 
      /**
       * Destruktor.
       * uwaga na pulapke ! prosze nie mylec ze tutaj powstaje wyciek pamieci bo nie usuwamy
       * objektow na ktore wstazuja wskazniki z vektora, objekty te sa tworzone (a co za tym idzie
       * i usuwane przez inna klase) tutaj sa przechowywane tylko wskazniki*/
      ~CUsedList(){}
      void use(CNode *x) {m_used.push_back(x);}
      bool was_used(CNode *x){
         for(int i=0;i<m_used.size();i++)
             if(m_used[i]==x) return true;
         return false;
      }
    protected:
      std::vector<CNode *> m_used; 
  };

/**
 * Klasa rozwiazujaca obwod.
 */  
class CCircuitSolver
{
public :
 /**
  * Konstruktor.
  * @param * data Wskaznik do obwodu ktory ma zostac rozwiazany.
  * @see ~CCircuitSolver();
  */  
  CCircuitSolver(CCircuit * data=NULL) {m_data=data;}
 /**
  * Destruktor.
  * @see CCircuitSolver();
  */    
  ~CCircuitSolver();
          
 /**
  * Rozwiazuje uklad.
  */  
  void solve();
 /**
  * Czysci uklad.
  */  
  void clear();
 /**
  * Wczytuje uklad.
  * @param *data wskaznik do obwodu wejsciowego.
  */  
  void load(CCircuit * data) {m_data=data;}
  
protected:  
  
  /** Wskaznik do obwodu wejsciowego */
  CCircuit *m_data;

  /** Lista przetworzonych elementow podczas procesu rozwiazywania */
  CUsedList used;
  
  /** Zwraca wskaznik do elementu o zadanej nazwie */
  CElement * get_item(std::string s);
          
  /** Wektor przetrzymujacy wszyskie wierzcholki rozszerzone */
  std::vector<CExNode *> exnodes;
  /** Wektor przetrzymujacy wszyskie galezie */
  std::vector<CBranch *> branches;
  
  /** Funkcja wykonuje zciaganie do wierzcholkow rzeczywistych (rozszerzonych) */
  void zciagaj();
  /** Wyszukuje wierzcholki, dodajac dowiazania do wezla ndarzednego */
  void serch_nodes(CExNode *des, CNode *start);
  /** Wypisuje wszyskie dowiazania symboliczne pomiedzy wierzcholkami a wiercholkiem nadrzednym */
  void print_aliases();
  /** TWorzy galezie */
  void make_branches();
  
  //omegi wystepujaca w calym obwodzie
  /** Wektor przetrzymujacy wszyskie czestosci kolowe wystepujace w obwodzie */
  std::vector<double> m_omegas;
  /** Funkcja odnajduje wszyskie czestosci kolowe wystepujace w obwodzie */
  void found_omegas();
  /** Funkcja wypisuje wszyskie czestosci kolowe wystepujace w obwodzie */
  void print_omegas();
  
  //macierze topologiczne
  /** Macierz incydencji A */
  CMatrix<Complex> *A;
  /** Macierz admitancji gaziowych Yb */
  CMatrix<Complex> *Yb;
  CMatrix<Complex> *Yn;
  /** Wymuszenia napiciowe stae E */
  CMatrix<Complex> *E;
  /** Wymuszenia prdowe stae I */
  CMatrix<Complex> *I;
  /** Wektor wydajnoci prdowych In */
  CMatrix<Complex> *In;
  /** Potencjay wezowe Un */
  CMatrix<Complex> *Un;
  
  //funkcje tworzace macierze
  /** Tworzy/wyznazca macierz incydencji A */
  void makeA();
  /** Tworzy/wyznazca macierz admitancji gaziowych Yb */
  void makeYb(double omega=0);
  /** Tworzy/wyznazca macierz admintacji wzowych Yn */
  void makeYn(double omega=0);
  /** Tworzy/wyznazca macierz mymuszen napiciowych stae E */
  void makeE(double omega=0);
  /** Tworzy/wyznazca wymuszenia prdowe stae I */
  void makeI(double omega=0);
  /** Tworzy/wyznazca wektor wydajnoci prdowych In */
  void makeIn();
  /** Tworzy/wyznazca potencjay wezowe Un */
  void makeUn();

};

#endif // CCIRCUITSOLVER_HH
