#ifndef CBranch_HH
#define CBranch_HH

#include <electro/CSignal.hh>
#include <electro/CElement.hh>


#include <math/Complex.hh>
#include <iostream>
#include <vector>


class CElement;
/**
 * Klasa reprezentuje galaz - czyli polaczenie szeregowe kilku elementow,
 * rozpoczynajacych sie i konczoacych na wezlach (rozszerzonych). Umozliwa
 * zwracanie wartosci impedancji, napiecia lub pradu danej gaezi w zaleznosci
 * od zadanej czestotliwosci.
 */
class CBranch
{
public:
   /**
    * Konstruktor.
    * @see ~CBranch();    
    */
    CBranch(){signal=NULL;}
   /**
    * Destruktor.
    * @see CBranch();
    */
    ~CBranch(){
       if(signal) delete signal;
     }
   /**
    * Funkcja zwraca wartosc impedancji.
    * @param omega czestosc katowa
    * @return wartosc impedancji
    */      
    Complex getZ(double omega);
   /**
    * Funkcja zwraca wartosc napiecia.
    * @param omega czestosc katowa
    * @return wartosc napiecia
    */  
    Complex getU(double omega);
   /**
    * Funkcja zwraca wartosc pradu.
    * @param omega czestosc katowa
    * @return wartosc pradu
    */  
    Complex getI(double omega);
   /**
    * Funkcja wyswietlajaca cala galaz na std::cout, napisana w celach
    * testowych.
    */  
    void print_branch();
   /**
    * Sprawdza czy galaz ma harakter pradowy.
    * @return TRUE - galaz pradowa
    *         FALSE - galaz napieciowa
    */  
    bool isCurent();
   /** Wskaznik do sygnalu pradowego przeplywajacego przez galaz */    
    CSignal *signal;
   /** Wektor wskaznikow do elementow bedacych w galezi */    
    std::vector<CElement *> elements;
   /** Numer pomocniczy , stosowany tylko w celach kontrolnych */        
    unsigned int nr;    
protected:
  int m_sign_u;
  int m_sign_i;
};


#endif

