#ifndef CCapasistor_HH
#define CCapasistor_HH

#include <electro/CElement.hh>
#include <vector>


/**
 * Klasa reprezentuje idealny kondensator.
 */
class CCapasistor : public CElementPasive
{
public:
   /**
    * Konstruktor.
    * @param name nazwa .
    * @param c pojemnosc.
    * @param x rzedna elementu na schemacie.
    * @param y odcieta elementu na schemacie.
    * @param rot rotacja elementu na schemacie.
    * @see ~CCapasistor();CElementPasive();
    */
    CCapasistor(std::string name,float c,int x=0,int y=0,int rot=0) : CElementPasive("CAP",name,x,y,rot) {
      m_unitSymbol  = "F"; 
      m_valueSymbol = "C";
      m_c=c;
    }
   /**
    * Destruktor.
    * @see CCapasistor;
    */      
    ~CCapasistor(){}

   /**
    * Ustawia wartosc pojemnosci.
    * @param c wartosc pojemnosci
    * @see setC();
    */      
    void setC(float c){m_c=c;}   
   /**
    * Funkcja zwraca wartosc pojemnosci.
    * @return pojemnosc
    * @see getC();
    */     
    float getC(void ){return m_c;}
    /**
     * Zwraca impedancje.
     * @param omega czestosc kolowa wymuszenia
     * @return wartosc impedancji
     * @see Complex getYc(double omega);
     */    
    Complex getZc(double omega){
     if(omega!=0)
       return Complex(0,-1/(omega*m_c));
     else
       return Complex(0,-MAX_DOUBLE);
     }
    /**
     * Zwraca konduktancje.
     * @param omega czestosc kolowa wymuszenia
     * @return wartosc konduktancji
     * @see Complex getZc(double omega);
     */        
    Complex getYc(double omega){return Complex(0,-omega*m_c);}
    
    
    double getValue() {return getC();}
    void draw();

protected: 
  /** Pojemnosc */
    float m_c;    
};

#endif
