#ifndef CCurent_HH
#define CCurent_HH

#include <electro/CElement.hh>
#include <vector>

class CElement;
class CElementActive;

/**
 * Klasa reprezentuje idealne zrodlo pradowe.
 */ 
class CCurent : public CElementActive
{
public:
   /**
    * Konstruktor.
    * @param name nazwa .
    * @param i wartosc pradu [A].
    * @param omega czestosc kolowa [Hz].
    * @param fi0 faza poczatkowa.
    * @param x rzedna elementu na schemacie.
    * @param y odcieta elementu na schemacie.
    * @param rot rotacja elementu na schemacie.
    * @see ~CCurent();CElementActive();
    */
   CCurent(std::string name,float i,double omega,double fi0,int x=0,int y=0,int rot=0) : CElementActive("CUR",name,omega,fi0,x,y,rot){
         m_i=i;
         for(int i=0;i<2;i++)
            pins.push_back(new CPin());
	 m_unitSymbol  = "A";
	 m_valueSymbol = "I";
    } 
   /**
    * Destruktor.
    * @see CCurent();
    */         
   ~CCurent() { }
   /**
    * Funkcja zwraca wartosc pradu.
    * @return wartosc pradu
    * @see getIc();
    */  
    float getI(void ){return m_i;}
   /**
    * Funkcja zwraca zespolona wartosc pradu.
    * @return wartosc pradu
    * @see getI();
    */  
    Complex getIc(void ){return Complex(m_i,m_fi0);}
    double getValue() {return getI();}
   /**
    * Ustawia wartosc pradu.
    * @param i wartosc pradu
    * @see getI();
    */  
    void setI(float i){m_i=i;}
    void draw();
protected: 
    //! amplituda praduv
    float m_i;
};
#endif
