#include <cmath>
#include <sstream>
#include <iostream>
#include <iomanip>

#include "CElement.hh"

#include <CCircuit.hh>

#include <networksarea.hh>


CElement::CElement(std::string symbol,std::string name,int x,int y,int rot){
	m_symbol=symbol;
	m_name=name;
	m_unitSymbol  = "";
	m_valueSymbol = "";
	m_x=x;
	m_y=y;
	m_rot=rot;
        branch=NULL;
	m_ParentCircuit = 0;
        signal=0;
	
}

CElement::~CElement(){
         for(int i=0;i<m_max_pin();i++)
            if(pins[i]) delete pins[i];
}

void CElement::print_conections(){
  std::cout <<"["<< m_name << "]";
  for(int i=0;i<pins.size();i++)
    if(pins[i]->element) 
      std::cout << "\t ("<<i<<")->" << (*(pins[i]->element)).getName()<< "." << pins[i]->pin;
  std::cout<<endl;
}

bool CElement::conect(CElement *dest,unsigned int src_pin,unsigned int dest_pin)
{
  if(src_pin<m_max_pin()){
    pins[src_pin]->element=dest;
    pins[src_pin]->pin=dest_pin;
    return true;
  }
  return false;
}


bool CElement::disconect(unsigned int src_pin)
{
  if(src_pin<m_max_pin()){
    pins[src_pin]->element=NULL;
    pins[src_pin]->pin=0;
    return true;
  }
  return false;
}


void CElement::draw()
{
  int squareSize = m_ParentCircuit->m_ParentDrawingArea->m_SquareSize;
  
  m_ParentCircuit->m_ParentDrawingArea->m_RefPixmap_NetworksArea->draw_line(
      m_ParentCircuit->m_ParentDrawingArea->m_RefGC_NetworksArea,
      0,0,
      m_x*squareSize,m_y*squareSize);

  // std::cout << "CElement::draw() called [" << m_x << "," << m_y << "," << m_rot << "]" << std::endl;
}


void CElement::drawConnections()
{
  RLC::CNetworksArea* parentNetworksArea = m_ParentCircuit->m_ParentDrawingArea;
  Glib::RefPtr<Gdk::Pixmap> refPixmap_NetworksArea = parentNetworksArea->m_RefPixmap_NetworksArea;
  Glib::RefPtr<Gdk::GC> refGC_NetworksArea = parentNetworksArea->m_RefGC_NetworksArea;
  int squareSize = parentNetworksArea->m_SquareSize;

  for (int i=0; i<m_max_pin(); ++i) {
    // std::cout << "Element = " << &(*(pins[i]->element)) << std::endl;
    if (pins[i]->element) {
      refPixmap_NetworksArea->draw_line(
	  refGC_NetworksArea,
	  getPinX(i)*squareSize,
	  getPinY(i)*squareSize,
	  ( (*(pins[i]->element)).getPinX(pins[i]->pin) )*squareSize,
	  ( (*(pins[i]->element)).getPinY(pins[i]->pin) )*squareSize);

      // std::cout << "CElement::drawConnections() called [" << getName() << "," << i << "] [" 
	        // << getPinX(i) << "," <<  getPinY(i) << ","
		// << (*(pins[i]->element)).getPinX(pins[i]->pin) << "," 
		// << (*(pins[i]->element)).getPinY(pins[i]->pin) << "]" << std::endl;
    }
  }
}


void CElement::drawName()
{
  RLC::CNetworksArea* parentNetworksArea = m_ParentCircuit->m_ParentDrawingArea;
  Glib::RefPtr<Gdk::Pixmap> refPixmap_NetworksArea = parentNetworksArea->m_RefPixmap_NetworksArea;
  Glib::RefPtr<Gdk::GC> refGC_NetworksArea = parentNetworksArea->m_RefGC_NetworksArea;
  Glib::RefPtr<Pango::Context> refContext_Main = parentNetworksArea->m_RefContext_Main;
  int squareSize = parentNetworksArea->m_SquareSize;
  
  Glib::RefPtr<Pango::Layout> nameText = Pango::Layout::create(refContext_Main);
  nameText->set_text(getName());
 
  // TODO: think about changing colors in functions. Do this here or outside, before
  // function call??
  refGC_NetworksArea->set_foreground(parentNetworksArea->m_Color_NameText);

  // TODO: rewrite this function draw text where no elements are placed. The best
  // way is to create virtual name drawing function for every element, cause
  // every of these is size-different.
  refPixmap_NetworksArea->draw_layout(
      refGC_NetworksArea,
      static_cast<int>(abs(m_x-0.6)*squareSize),
      static_cast<int>(abs(m_y-0.5)*squareSize),
      nameText);

  refGC_NetworksArea->set_foreground(parentNetworksArea->m_Color_Element);

  // std::cout << "CElement:drawName() called" << std::endl; 

}


void CElement::drawValue()
{
  RLC::CNetworksArea* parentNetworksArea = m_ParentCircuit->m_ParentDrawingArea;
  Glib::RefPtr<Gdk::Pixmap> refPixmap_NetworksArea = parentNetworksArea->m_RefPixmap_NetworksArea;
  Glib::RefPtr<Gdk::GC> refGC_NetworksArea = parentNetworksArea->m_RefGC_NetworksArea;
  Glib::RefPtr<Pango::Context> refContext_Main = parentNetworksArea->m_RefContext_Main;
  int squareSize = parentNetworksArea->m_SquareSize;
  
  Glib::RefPtr<Pango::Layout> nameText = Pango::Layout::create(refContext_Main);

  // CElement* pce = this; // to have w virtual getValueString calling
  nameText->set_text(getValueString());
 
  // TODO: think about changing colors in functions. Do this here or outside, before
  // function call??
  refGC_NetworksArea->set_foreground(parentNetworksArea->m_Color_ValueText);

  // TODO: rewrite this function to draw text where no elements are placed. The best
  // way is to create virtual name drawing function for every element, cause
  // every of these is size-different. Or maybe create a two dimesional array
  // of every field as a pointer to CElement, to know if the field is occupied
  // by any element - this seems to be a good way.
  refPixmap_NetworksArea->draw_layout(
      refGC_NetworksArea,
      static_cast<int>(abs(m_x-0.7)*squareSize),
      static_cast<int>(abs(m_y-0.0)*squareSize),
      nameText);

  refGC_NetworksArea->set_foreground(parentNetworksArea->m_Color_Element);

  // std::cout << "CElement:drawValue() called" << std::endl; 

}


std::string CElement::getValueString()
{
  // value output stream
  // TODO: do we really have to do this in three lines?? In would be great to have
  // one only.
  std::ostringstream vos;
  // TODO: add precision values to @FUTUR_CONFIG@
  vos << getValueSymbol() << "=" << std::setprecision(2) << getValue() << " " << getUnitSymbol();
 
  return vos.str();
}


int CElement::getPinX(int pinNumber)
{
  // pinNumber is either 0 (front of element) or 1 (back of element).

  //TODO: change this below to not use 'case'. Figure out some simple math formula for this.
  // Setting variables based on rotatation for futher use.
  int rot_x = 0, rot_y = 0;
  switch (m_rot) {
    case  1: rot_y = 1; break;
    case  2: rot_x =-1; break;
    case  3: rot_y =-1; break;
    default: rot_x = 1; 
  }
  

  return (m_x+1+(2-4*pinNumber)*rot_x);
}


int CElement::getPinY(int pinNumber)
{
  // pinNumber is either 0 (front of element) or 1 (back of element).

  //TODO: change this below to not use 'case'. Figure out some simple math formula for this.
  // Setting variables based on rotatation for futher use.
  int rot_x = 0, rot_y = 0;
  switch (m_rot) {
    case  1: rot_y = 1; break;
    case  2: rot_x =-1; break;
    case  3: rot_y =-1; break;
    default: rot_x = 1; 
  }
  
  return (m_y+1-(2-4*pinNumber)*rot_y);
}


void CNode::draw()
{
  RLC::CNetworksArea* parentNetworksArea = m_ParentCircuit->m_ParentDrawingArea;
  Glib::RefPtr<Gdk::Pixmap> refPixmap_NetworksArea = parentNetworksArea->m_RefPixmap_NetworksArea;
  Glib::RefPtr<Gdk::GC> refGC_NetworksArea = parentNetworksArea->m_RefGC_NetworksArea;
  int squareSize = parentNetworksArea->m_SquareSize;
  int nodeSize   = parentNetworksArea->m_NodeSize;

  refGC_NetworksArea->set_foreground(parentNetworksArea->m_Color_Node);
  // TODO: create a mf_drawNode(x,y) method in networksarea to be used by this function, and by any other
  // that will draw a node.
  refPixmap_NetworksArea->draw_arc(
      refGC_NetworksArea,
      1,
      m_x*squareSize-nodeSize,
      m_y*squareSize-nodeSize,
      2*nodeSize,
      2*nodeSize,
      0,360*64);

  refGC_NetworksArea->set_foreground(parentNetworksArea->m_Color_Element);

  // std::cout << "CNode:draw() called [" << m_x << "," << m_y << "]" << std::endl;
  // std::cout << "CNode:draw() called [" << (m_x*squareSize-nodeSize) << "," << (m_y*squareSize-nodeSize) << "]" << std::endl;
}


void CNode::drawName()
{
 // This is empty to not draw nodes names. 
}


void CNode::drawValue()
{
 // This is empty to not draw nodes names. 
}


std::string CElementControl::getValueString()
{
  // value output stream
  // TODO: do we really have to do this in three lines?? In would be great to have
  // one only.
  std::ostringstream vos;
  // TODO: add precision values to @FUTUR_CONFIG@
  vos << std::setprecision(2) << getGm() << "*" << getValueSymbol() << "(" << getOn() << ")";
 
  return vos.str();
}

