#ifndef CInductor_HH
#define CInductor_HH

#include <electro/CElement.hh>
#include <vector>
/**
 * Klasa reprezentuje idealna indukcyjnosc (cewka, zwojnica).
 */
class CInductor : public CElementPasive
{
public:
   /**
    * Konstruktor.
    * @param name nazwa .
    * @param l indukcyjnosc.
    * @param x rzedna elementu na schemacie.
    * @param y odcieta elementu na schemacie.
    * @param rot rotacja elementu na schemacie.
    * @see ~CInductor();CElementPasive();
    */
    CInductor(std::string name,float l,int x=0,int y=0,int rot=0) : CElementPasive("IND",name,x,y,rot) {
      m_unitSymbol  = "H";
      m_valueSymbol = "L";
      m_l=l;
    }
   /**
    * Destruktor.
    * @see CInductor;
    */    
    ~CInductor(){}
    
   /**
    * Ustawia wartosc indukcyjnosci.
    * @param l wartosc indukcyjnosci
    * @see setL();
    */      
    void setL(float l){m_l=l;}   
   /**
    * Funkcja zwraca wartosc indukcyjnosci.
    * @return indukcyjnosc
    * @see getL();
    */
    float getL(void ){return m_l;}    
    /**
     * Zwraca impedancje.
     * @param omega czestosc kolowa wymuszenia
     * @return wartosc impedancji
     * @see Complex getYc(double omega);
     */    
    Complex getZc(double omega){return Complex(0,(omega*m_l));}
    /**
     * Zwraca konduktancje.
     * @param omega czestosc kolowa wymuszenia
     * @return wartosc konduktancji
     * @see Complex getZc(double omega);
     */    
    Complex getYc(double omega){
     if(omega!=0)
       return Complex(0,1/(omega*m_l));
     else
       return Complex(0,MAX_DOUBLE);
     }

    void draw();
protected: 
    /** Indukcyjnosc */
    float m_l;
};

#endif
