#ifndef CResistor_HH
#define CResistor_HH

#include <electro/CElement.hh>
#include <vector>

/**
 * Klasa reprezentuje idealny rezystor.
 */
class CResistor : public CElementPasive
{
public:
   /**
    * Konstruktor.
    * @param name nazwa .
    * @param r rezystancja.
    * @param x rzedna elementu na schemacie.
    * @param y odcieta elementu na schemacie.
    * @param rot rotacja elementu na schemacie.
    * @see ~CResistor();CElementPasive();
    */
    CResistor(std::string name,float r,int x=0,int y=0,int rot=0) : CElementPasive("RES",name,x,y,rot)  {
         m_r=r;
         for(int i=0;i<2;i++)
            pins.push_back(new CPin());
	 m_unitSymbol  = "Ohm";
	 m_valueSymbol = "R";
    }
   /**
    * Destruktor.
    * @see CResistor;
    */    
    ~CResistor(){}
   /**
    * Ustawia wartosc rezystancji.
    * @param r wartosc rezystancji
    * @see setR();
    */     
    void setR(float r){m_r=r;}
   /**
    * Funkcja zwraca wartosc rezystancji.
    * @return rezystancja
    * @see getR();
    */
    float getR(void ){return m_r;}
    
    /**
     * Zwraca impedancje.
     * @param omega czestosc kolowa wymuszenia
     * @return wartosc impedancji
     * @see Complex getYc(double omega);
     */
    Complex getZc(double omega){return Complex(m_r,0);}
    /**
     * Zwraca konduktancje.
     * @param omega czestosc kolowa wymuszenia
     * @return wartosc konduktancji
     * @see Complex getZc(double omega);
     */    
    Complex getYc(double omega){return Complex(1/m_r,0);}
    
    
    double getValue() {return getR();}
    void draw();
protected: 
    /** Rezystencja */
    float m_r;
};
#endif

