#ifndef CSignal_HH
#define CSignal_HH

#include <vector>
#include <cmath>
#include <sstream>

namespace RLC {
  class CNetworksArea;
}
/**
 * W pelni abstrakcyjna klasa bazowa reprezentujaca fale
 */
class CWave{
public:
     virtual ~CWave(){};
     virtual double time(double t)=0;
};
/**
 * Klasa reprezentujaca fale sinusoidalna o parametrach stalych w czasie
 */
class CSinWave:public CWave{
  protected:
     double m_omega;
     double m_amplitude;
     double m_fi0;
  public:
     CSinWave(double a, double omega, double fi0):m_amplitude(a),m_omega(omega),m_fi0(fi0){};
     double time(double t){
       return m_amplitude * sin(m_omega*t+m_fi0);
     }
     void setAmplitude(double a){m_amplitude=a;}
     void setOmega(double o){m_omega=o;}
     void setFi(double fi0){m_fi0=fi0;}
     double getAmplitude(){return m_amplitude;}
     double getOmega(){return m_omega;}
     double getFi(){return m_fi0;}
};
/** 
 * ta klasa jest w sumie nie potrzebna, jednak zostala stworzona dla wiekszej przejrzystosci kodu
 * rownie dobrze mozna by dac CSinWave(a,0,pi/2) jednak to by prowadzilo do przechowywania wiekszej
 * ilosci danych jak rownierz bylo mniej oczywiste dla czytajacego
*/
class CLineWave:public CWave{
  protected:
     double m_amplitude;
  public:
     CLineWave(double a):m_amplitude(a){};
     double time(double t){
       return m_amplitude;
     }
     void setAmplitude(double a){m_amplitude=a;}
     double getAmplitude(){return m_amplitude;}     
};

/** 
 * Klasa CSignal bedzie mogla przechowywac informacje o syglanach, 
 * kazdy sygnal bedzie reprezentowany jako super pozycja dowolnych sygnalow sinusoidalnych
 * (dowolna amplituda , faza i czestos)
*/
class CSignal
{
protected: 
    std::vector<CWave *> m_waves;
    std::string m_description;
    std::string m_unit;
    
public:
    CSignal(std::string text="",std::string unit=""):m_description(text),m_unit(unit) {};
    ~CSignal(){
      for(int i=0;i<m_waves.size();++i)
        delete m_waves[i];
    };
    void addWave(double a, double omega, double fi0){
      if(omega!=0)
        m_waves.push_back(new CSinWave(a,omega,fi0));
      else
        m_waves.push_back(new CLineWave(a));
    };
    double time(double t){
      double super_position=0;
      for(int i=0;i<m_waves.size();++i)
         super_position+=m_waves[i]->time(t);
      return super_position;   
    }
    void clear(){
      for(int i=0;i<m_waves.size();++i)
        delete m_waves[i];
    }
    std::string getDescription(){return m_description;}
    std::string getText(){return m_description;}
    std::string getUnit(){return m_unit;}
    std::string getEq(){
       std::string tmp;
       std::stringstream x;
       for(int i=0;i<m_waves.size();++i)
         {
          CSinWave *p;
          CLineWave *q;
          if(p=dynamic_cast<CSinWave *> (m_waves[i]))
             x<<p->getAmplitude()<<"*sin("<<p->getOmega()<<"t+"<<p->getFi()<<")";
          else if(q=dynamic_cast<CLineWave *> (m_waves[i]))
             x<<q->getAmplitude();
          x<<"+";
         }
         x>>tmp;
       return tmp;
    }
};

#endif
