#ifndef CVoltage_HH
#define CVoltage_HH

#include <electro/CElement.hh>
#include <vector>
class CElement;
class CElementActive;
/**
 * Klasa reprezentuje idealne zrodlo napieciowe.
 */ 
class CVoltage : public CElementActive
{
public:
   /**
    * Konstruktor.
    * @param name nazwa .
    * @param u wartosc napiecia [A].
    * @param omega czestosc kolowa [Hz].
    * @param fi0 faza poczatkowa.
    * @param x rzedna elementu na schemacie.
    * @param y odcieta elementu na schemacie.
    * @param rot rotacja elementu na schemacie.
    * @see ~CVoltage();CElementActive();
    */
    CVoltage(std::string name,float u,double omega,double fi0,int x=0,int y=0,int rot=0) : CElementActive("VOL",name,omega,fi0,x,y,rot){
         m_u=u;
         for(int i=0;i<2;i++)
            pins.push_back(new CPin());
	 m_unitSymbol  = "V";
	 m_valueSymbol = "U";
    }
   /**
    * Destruktor.
    * @see CVoltage();
    */        
    ~CVoltage(){}
   /**
    * Funkcja zwraca wartosc napiecia.
    * @return wartosc napiecia
    * @see getUc();
    */      
    float getU(void ){return m_u;}
   /**
    * Funkcja zwraca wartosc zespolona napiecia.
    * @return wartosc napiecia
    * @see getU();
    */
    Complex getUc(void ){return Complex(m_u,m_fi0);}
    double getValue() {return getU();}
   /**
    * Ustawia wartosc napiecia.
    * @param u nowa wartosc napiecia
    * @see getI();
    */ 
    void setU(float u){m_u=u;}
    void draw();

    // TODO: change these functions to one that returns Glib::Point.
    int getPinX(int pinNumber);
    int getPinY(int pinNumber);
protected: 
    //! wartosc napiecia
    float m_u;
};
#endif

