#include <math/CMatrix.hh>
#include <strstream>
  
#define LOOP(L) {int l = m_columns * m_rows; for (int i = 0; i < l; ++i) L;}

//default constructor without any arguments
template <typename Type> 
CMatrix<Type>::CMatrix() : m_columns(0) , m_rows(0) , m_data(0) {}

//constructor 
template <typename Type> 
CMatrix<Type>::CMatrix(int m, int n)
  : m_rows(m)
  , m_columns(n)
  , m_data(new Type [n * m])
  {}
  
//copy`s constructor 
template <typename Type> 
CMatrix<Type>::CMatrix(const CMatrix<Type>& x) 
 {
   m_columns = x.m_columns;
   m_rows = x.m_rows;
   m_data = new Type [m_columns * m_rows];
   LOOP(m_data[i] = x.m_data[i])
 }

//destructor
template <typename Type> 
CMatrix<Type>::~CMatrix() 
 {
   delete [] m_data;
 }
 

template <typename Type> 
void CMatrix<Type>::zero()
 { 
   LOOP(m_data[i] = 0) 
 }
 
template <typename Type> 
CMatrix<Type>& CMatrix<Type>::operator=(const CMatrix<Type>& x) 
 {
   //check if we do not have to copy the same object
   if (this == &x) return *this;
   //save new size
   m_columns = x.m_columns;
   m_rows = x.m_rows;
   //delete old datas
   delete [] m_data;
   //create new table
   m_data = new Type [m_columns * m_rows];
   //rewrite data
   LOOP(m_data[i] = x.m_data[i])
   return *this;
 }

template <typename Type> 
CMatrix<Type>& CMatrix<Type>::operator+=(Type x) 
 {
   LOOP(m_data[i] += x) 
   return *this;
 }

template <typename Type> 
CMatrix<Type>& CMatrix<Type>::operator-=(Type x) 
 {
   LOOP(m_data[i] -= x) 
   return *this;
 }

template <typename Type> 
CMatrix<Type>& CMatrix<Type>::operator*=(Type x) 
 {
   LOOP(m_data[i] *= x) 
   return *this;
 }

template <typename Type> 
CMatrix<Type>& CMatrix<Type>::operator/=(Type x) 
 {
   return *this *= (Type)1 / x;
 }

template <typename Type> 
CMatrix<Type>& CMatrix<Type>::operator+=(const CMatrix& x) 
 { 
   if(m_rows!=x.rows() || m_columns!=x.columns()) throw CMatrixSizeError();
   LOOP(m_data[i] += x.m_data[i]) 
   return *this; 
 }
   
template <typename Type> 
CMatrix<Type>& CMatrix<Type>::operator-=(const CMatrix<Type>& x) 
 { 
   if(m_rows!=x.rows() || m_columns!=x.columns()) throw CMatrixSizeError();
   LOOP(m_data[i] -= x.m_data[i]) 
   return *this; 
 }
   
template <typename Type> 
CMatrix<Type> CMatrix<Type>::operator*(Type x) const 
 {
   CMatrix<Type> tmp = *this; 
   return tmp *= x;
 }

template <typename Type> 
CMatrix<Type> CMatrix<Type>::operator/(Type x) const 
 {
   CMatrix<Type> tmp = *this; 
   return tmp /= x;
 }

template <typename Type> 
CMatrix<Type> CMatrix<Type>::operator+(const CMatrix<Type>& x) const 
 {
   CMatrix<Type> tmp = *this; 
   return tmp += x;
 }
   
template <typename Type> 
CMatrix<Type> CMatrix<Type>::operator-(const CMatrix<Type>& x) const 
 {
   CMatrix<Type> tmp = *this; 
   return tmp -= x;
 }
   
template <typename Type> 
CMatrix<Type> CMatrix<Type>::operator*(const CMatrix<Type>& x) const 
 { 
   if(x.rows()!=m_columns) throw CMatrixSizeError();
   CMatrix<Type> tmp(m_rows, x.m_columns);
   for (int i = 0; i < tmp.m_rows; ++i) {
     for (int j = 0; j < tmp.m_columns; ++j) {
       tmp[i][j] = 0;
       for (int k = 0; k < m_columns; ++k) {
         tmp[i][j] += (m_data[i*m_rows + k]) * (x[k][j]);
       }
     }
   }
   return tmp;
 }
 

template <typename Type> 
void CMatrix<Type>::invert()
{
   CMatrix<Type> A(m_rows, m_columns);
   for (int i = 0; i < m_rows; ++i) 
     for (int j = 0; j < m_columns; ++j) 
       A[i][j] = i == j ? 1 : 0;
   CMatrix<Type>& M = *this;
   for (int i = 0; i < m_columns; ++i) {
     Type s = (Type)1 / M[i][i];
     for (int k = 0; k < m_columns; ++k) {
       A[i][k] = A[i][k] * s;
       M[i][k] = M[i][k] * s;
     }
     for (int j = 0; j < m_rows; ++j) {
       if (j == i) continue;
       Type t = M[j][i];
       for (int k = 0; k < m_columns; ++k) {
         A[j][k] = A[j][k] - A[i][k] * t;
         M[j][k] = M[j][k] - M[i][k] * t;
       }
     }
   }
   *this = A;
 }
 
template <typename Type> 
CMatrix<Type> CMatrix<Type>::T() const
 {
   const CMatrix<Type>& M = *this;
   CMatrix<Type> A(m_columns, m_rows);
   for (int i = 0; i < m_rows; ++i) {
     for (int j = 0; j < m_columns; ++j) {
       A[j][i] = M[i][j];
     }
   }
   return A;
 }
 
template <typename Type> 
CMatrix<Type> CMatrix<Type>::operator-() const 
 {
   CMatrix<Type> A(m_rows, m_columns); 
   LOOP(A.m_data[i] = -this->m_data[i])
   return A;
 }
 
template <typename Type> 
void CMatrix<Type>::print(std::ostream& os) const
 {
   const CMatrix<Type>& x = *this;
   for (int i = 0; i < m_rows; ++i) {
     cout << "| ";
     for (int j = 0; j < m_columns; ++j) {
       os << x[i][j] << "\t ";
     }
     os <<" |"<< std::endl;
   }
}
 
template class CMatrix<float>;
template class CMatrix<signed char>;
template class CMatrix<Complex>;

