#ifndef CMATRIX_HH
#define CMATRIX_HH
 
#include <iostream>
#include <math/Complex.hh>
#include <CRlcException.hh>

template <typename Type>
class CMatrix
{
public:
 CMatrix();
 CMatrix(int rows, int columns);
 CMatrix(const CMatrix<Type>& m);
 ~CMatrix();
 
 void zero();
 Type* operator[](int i);
 const Type* operator[](int i) const;
 int rows() const {return m_rows;}
 int columns() const {return m_columns;}

 void setRows(int r) {m_rows=r;}
 
 void invert();

 CMatrix<Type>& operator=(const CMatrix<Type>& x);
 CMatrix<Type>& operator+=(Type x);
 CMatrix<Type>& operator-=(Type x);
 CMatrix<Type>& operator*=(Type x);
 CMatrix<Type>& operator/=(Type x);
 CMatrix<Type>& operator+=(const CMatrix<Type>& x);
 CMatrix<Type>& operator-=(const CMatrix<Type>& x);
 
 CMatrix<Type> operator*(Type x) const;
 CMatrix<Type> operator/(Type x) const;
 CMatrix<Type> operator+(const CMatrix<Type>& x) const;
 CMatrix<Type> operator-(const CMatrix<Type>& x) const;
 CMatrix<Type> operator*(const CMatrix<Type>& x) const;
 CMatrix<Type> operator-() const;
 CMatrix<Type> T() const; // transpose
  
 void print(std::ostream& os = std::cout) const;
 
 class CMatrixException:public CRlcException
 {
   public:
     CMatrixException(const std::string &e="Matrix Exception"):CRlcException(e){};
 };
 
 class CMatrixSizeError:public CMatrixException
 {
   public:
     CMatrixSizeError(const std::string &e="Matrix Size Error"):CMatrixException(e){};
 };
 

private:
 
 int m_rows, m_columns; //number of rows and colums
 Type* m_data; //pointer to data
  
};

template <typename Type>
inline Type* CMatrix<Type>::operator[](int i) {return m_data + m_columns * i;}

template <typename Type>
inline const Type* CMatrix<Type>::operator[](int i) const {return m_data + m_columns * i;}

template <typename Type>
inline CMatrix<Type> operator*(float x, const CMatrix<Type>& y) {return y * x;}
 
#endif 
