#include <math/Complex.hh>

double Complex::_sin(double x){
	if(x>PI_2)
		return sin(PI-x);
	else	
		return sin(x);
}
double Complex::_cos(double x){
		return cos(x);
}

Complex::Complex(float re,float im){
	Re=re;
	Im=im;
}

float Complex::getRe() { 
	return Re; 
}

float Complex::getIm() { 
	return Im; 
}

float Complex::getM(){
	return sqrt(Re*Re + Im*Im);
}

float Complex::getFi(){
   if (Re==0 )
		return (Im>0) ? 0 : M_PI;
   else 	
		return (atan(Im/Re) + ((Re<0) ? ((Im<0) ? -M_PI : M_PI): 0));
}


void Complex::set(float re,float im){
	Re=re;
	Im=im;
}

void Complex::sprzez(){
	Im=-Im;
}

Complex Complex::pierw(){
    float a,b;
    a=sqrt(getM())*_cos(getFi()/2);
    b=sqrt(getM())*_sin(getFi()/2);
    return Complex(a,b);
}

Complex Complex::sprzezona(){
	return Complex(Re,-Im);
}
Complex Complex::operator * (const Complex f){
	return Complex((Re*f.Re) - (Im * f.Im),(Re*f.Im) +(Im*f.Re ));
}
Complex Complex::operator + (const Complex f){
	return Complex(Re+f.Re,Im+f.Im);
}
Complex Complex::operator - (const Complex f){
	return Complex(Re-f.Re,Im-f.Im);
}
Complex Complex::operator / (const Complex f){
	if(f.Re == 0 && f.Im == 0) {
		throwError("Dzielenie przez zero !");
		return Complex(0,0);
	}
	return Complex( ((Re*f.Re) + (Im * f.Im))/ (f.Re*f.Re + f.Im*f.Im) ,
                    ((Im*f.Re) - (Re*f.Im))  / (f.Re*f.Re + f.Im*f.Im));
}
Complex Complex::operator - (){
        return Complex(-Re,-Im);
}

Complex Complex::operator *= (const Complex f){
        return *this=*this * f;
}
Complex Complex::operator += (const Complex f){
        return *this=*this + f;
}
Complex Complex::operator -= (const Complex f){
        return *this=*this - f;
}
Complex Complex::operator /= (const Complex f){
        return *this=*this / f;
}



bool Complex::operator == (Complex f){
	if(Re==f.Re && Im==f.Im) 
	 	return true;
	 else 
	 	return false;
}

bool Complex::operator != (Complex f){
        if(Re!=f.Re || Im!=f.Im) 
                return true;
         else 
                return false;
}


Complex Complex::operator * (float f){
        return Complex((Re*f),(Im*f));
}
Complex Complex::operator + (float f){
        return Complex(Re+f,0);
}
Complex Complex::operator - (float f){
        return Complex(Re-f,0);
}
Complex Complex::operator / (float f){
        if(f == 0) {
                throwError("Dzielenie przez zero !");
                return Complex(0,0);
        }
        return Complex( (Re/f) , (Im/f));
}

bool Complex::operator == (float f){
        if(Re==f && Im==0) 
                return true;
         else 
                return false;
}	

ostream& operator<<(ostream& s, Complex z) {
	if(z.Re!=0)
	    s << z.Re ;
	if(z.Im!=0)
	    s << ((z.Im>0) ? "+" : "") << z.Im <<"j";
	    
	if(z.Re==0 && z.Im==0)	    
		s << 0;
	return  s;
}


istream& operator>>(istream& s, Complex& z) {
  char c;
  double a, b;
  //wczytujemy pierwsz liczbe
  s >> a;
  //jesli sa jakies znaki biae to je wycinamy  
  do{
    c=s.peek();
    if(c==' ' || c=='\t') s.get();
  }  
  while (c==' ' || c=='\t');
  
  //teraz badamy nie bialy znak
  switch(c){
	  case '+': s.get();
	  			s >> b >> c;
	      		if (c == 'j') 
      				z = Complex(a,b);
    			break;
	  case '-': s.get();
	  			s >> b >> c;
	      		if (c == 'j') 
      				z = Complex(a,-b);
    			break;
	  case 'j': s.get();
	  			z = Complex(0,a);
    			break;
	  default : z = Complex(a);
    			break;
	  			
  }  
  return s;
} 

void Complex::throwError(const string& err) const
{
	std::ostringstream lStream;
	lStream << "[Complex] :: " << err<<endl;
	throw runtime_error(lStream.str());
}
