/*	
*  Parser.hh	
*	
*  Created by Marc Parizeau on Sat Oct 26 2002.	
*  Copyright  2002 LVSN. All rights reserved.	
*	
*/	
	
#ifndef XML_Parser_hh_	
#define XML_Parser_hh_	
	
#include <xml/Streamer.hh>
#include <xml/Tokenizer.hh>
#include <map>	
	
namespace XML {	
	
class Parser	
{	
public:	

	class Node {	
	public:	
		enum Type{	
			eDecl,	
			eSpecial,	
			eTag,	
			eString	
		};	
	
		Node(void) : mChild(0), mSibling(0) {}
		~Node(void);
	
		Node* getFirstChild() const {return mChild;}	
		Node* getNextSibling() const {return mSibling;}	
		std::string getTagAttribute(const std::string& inName) const;	
		const std::string& getTagName() const {return mValue;}	
		Type getType() const {return mType;}	
		std::string getValue() const {return mValue;}	
		void serialize(Streamer& outStream, bool inIndent=true) const;	
	
	protected:	
		Type mType;	
		std::string mValue;	
		std::map<std::string,std::string> mAttributes;	
		Node* mChild;	
		Node* mSibling;	
	
		static Node* parse(Tokenizer& inTokenizer);	
		void parseAttributeList(Tokenizer& inTokenizer, std::string& outToken);	
		void parseStartTag(Tokenizer& inTokenizer, std::string& outToken);	
		void throwError(Tokenizer& inTokenizer, const std::string& inMessage) const;	
	
	friend class Parser;
		//friend class Chroma;
	
	private:
		Node(const Node&);		
		void operator=(const Node&);	
	};//Node

public:
	Parser(void) : mRoot(0) {}
	~Parser(void) {if(mRoot) delete mRoot;}

	Node* getRootNode() const {return mRoot;}
	
	void parse(const std::string& inFileName);	
	void parse(std::istream& inStream, const std::string& inName="");

protected:	
	Node* mRoot;	


private:	
	// Restrict (disable) copy constructor.	
	Parser(const Parser&);	
	void operator=(const Parser&);	
}; //end class parser
	
};// namespace XML	
	
#endif // XML_Parser_hh_	


