#ifndef XML_Streamer_hh_
#define XML_Streamer_hh_
 
#include <iostream>
#include <string>
#include <stack>

namespace XML {

class Streamer {
public:
Streamer(std::ostream& inStream, unsigned int inWidth=2) : mStream(inStream), mWidth(inWidth), mClosed(true) {}

void closeTag(bool inIndent=true);
       std::ostream& getStream();
       void insertAttribute(const std::string& inName, double inValue);
       void insertAttribute(const std::string& inName, std::string inValue);
      template<class Container>
          void insertContainer(const Container& inValue, const std::string& inName="");
       void insertPrimitiveTag(double invalue, const std::string& inTagName="Float", const std::string& inAttributeName="value");
       void insertPrimitiveTag(int invalue, const std::string& inTagName="Integer", const std::string& inAttributeName="value");
       void insertPrimitiveTag(const std::string& invalue, const std::string& inTagName="String", const std::string& inAttributeName="value");
       void insertSpecialTag(const std::string& inTag);
       void openTag(const std::string& inName, bool inIndent=true);
       
       Streamer& operator<<(int inValue);
       Streamer& operator<<(double inValue);
       Streamer& operator<<(const std::string& inValue);
 
       template<class T> Streamer& operator<<(const T& inClass) {
          inClass.write(*this);
          return *this;
       }
       
     protected:
       std::ostream& mStream; 
       std::stack<std::string> mTags; 
       unsigned int mWidth; 
       bool mClosed; 
       
    };
 
    template<class Container>
       void Streamer::insertContainer(const Container& inContainer, 
                                      const std::string& inName 
                                      )
    {
       // insert optional tag name
       if(inName != "") openTag(inName);
       // iterate through container
       for(typename Container::const_iterator i = inContainer.begin(); i != inContainer.end(); ++i)
       {
          (*this) << (*i);
       }
       if(inName != "") closeTag();
    }
 
 } // XML namespace
 
 #endif // XML_Streamer_hh_
