#ifndef XML_Tokenizer_hh_
#define XML_Tokenizer_hh_

#include <iostream>
#include <string>
#include <set>
#include <stdlib.h>

namespace XML {

class Tokenizer {
public:
      Tokenizer(void) : mLine(1), mStream(0), mWhiteSpace(" \t\n\r") {}
      Tokenizer(std::istream& inStream) : mLine(1), mStream(&inStream), mWhiteSpace(" \t\n\r") {}
 
      std::string getNextToken();
       bool getNextToken(std::string& outToken);
 
       void addStringToken(const std::string& inToken) {mStringTokens.insert(inToken);}
       void clearStringTokens() {mStringTokens.clear();}
      unsigned int getLineNumber() {return mLine;}
       std::istream* getStreamPtr() {return mStream;}
      const std::string& getStreamName() const {return mName;}
      const std::string& getSingleCharTokens() const {return mSingleCharTokens;}
       const std::string& getWhiteSpace() const {return mWhiteSpace;}
      void removeStringToken(const std::string& inToken) {mStringTokens.erase(inToken);}
      void setLineNumber(unsigned int inLine) {mLine = inLine;}
       void setStream(std::istream& inStream) {mStream = &inStream;}
      void setStreamName(const std::string& inName) {mName = inName;}
      void setSingleCharTokens(const std::string &inTokens) {mSingleCharTokens = inTokens;}
       void setWhiteSpace(const std::string &inWhite) {mWhiteSpace = inWhite;}
 
     protected:
       unsigned int mLine; 
       std::string mName; 
       std::istream* mStream; 
       std::string mWhiteSpace; 
       std::string mSingleCharTokens; 
       std::set<std::string> mStringTokens; 

};

 inline int convertStringToInteger(const std::string& inString) {return atoi(inString.c_str());}
 inline double convertStringToFloat(const std::string& inString) {return atof(inString.c_str());}

} 

#endif // XML_Tokenizer_hh_


