/*
 * RolLerCoaster Linear Networks Solver
 *
 * RolLerCoaster is the legal property of its developers, whose names are 
 * listed in the COPYRIGHT file distributed with this source distribution.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the free software foundation; either version 2 of the license, or
 * (at your option) any later version.
 *
 * this program is distributed in the hope that it will be useful,
 * but without any warranty; without even the implied warranty of
 * merchantability or fitness for a particular purpose.  see the
 * gnu general public license for more details.
 *
 * you should have received a copy of the gnu general public license
 * along with this program; if not, write to the free software
 * foundation, inc., 59 temple place, suite 330, boston, ma  02111-1307  usa
 *
 */


#include <gtkmm/stock.h>

#include "mainmenubar.hh"
#include "mainwindow.hh"
#include "networksarea.hh"
#include <aboutdialog.hh>

using namespace RLC;


CMainMenuBar::CMainMenuBar()
{

  /*
   * Filling the menus. Adding elements and connecting them with 
   * their activation signal handlers.
   */

  // "File" menu:
  {
    Gtk::Menu::MenuList& menuList = m_Menu_File.items();

    // menuList.push_back( Gtk::Menu_Helpers::StockMenuElem( Gtk::Stock::NEW, Gtk::AccelKey("<control>n"),
	// sigc::mem_fun(*this, &CMainMenuBar::on_menu_file_new) ) );
    menuList.push_back( Gtk::Menu_Helpers::StockMenuElem( Gtk::Stock::OPEN, Gtk::AccelKey("<control>o"),
	sigc::mem_fun(*this, &CMainMenuBar::on_menu_file_open) ) );
    menuList.push_back( Gtk::Menu_Helpers::StockMenuElem( Gtk::Stock::QUIT, Gtk::AccelKey("<control>q"),
	sigc::mem_fun(*this, &CMainMenuBar::on_menu_file_quit) ) );
  }

  // "Edit" menu:
  // {
    // Gtk::Menu::MenuList& menuList = m_Menu_Edit.items();
    // 
    // menuList.push_back( Gtk::Menu_Helpers::StockMenuElem( Gtk::Stock::PREFERENCES,
	// sigc::mem_fun(*this, &CMainMenuBar::on_menu_edit_preferences) ) );
  // }

  // "Network" menu:
  {
    Gtk::Menu::MenuList& menuList = m_Menu_Network.items();

    // menuList.push_back( Gtk::Menu_Helpers::MenuElem( "_Solve", Gtk::AccelKey("<control>F9"),
    // TODO: Using "execute" stock item for a while - change this to your own
    // "solve" item.
    menuList.push_back( Gtk::Menu_Helpers::StockMenuElem( Gtk::Stock::EXECUTE, Gtk::AccelKey("<control>F9"),
	sigc::mem_fun(*this, &CMainMenuBar::on_menu_network_solve) ) );
  }

  // "Help" menu:
  {
    Gtk::Menu::MenuList& menuList = m_Menu_Help.items();

    menuList.push_back( Gtk::Menu_Helpers::MenuElem( "_About",
	sigc::mem_fun(*this, &CMainMenuBar::on_menu_help_about) ) );
  }

  // Adding menus to menu bar of CMainMenuBar class:
  items().push_back( Gtk::Menu_Helpers::MenuElem( "_File", m_Menu_File) );
  // items().push_back( Gtk::Menu_Helpers::MenuElem( "_Edit", m_Menu_Edit) );
  items().push_back( Gtk::Menu_Helpers::MenuElem( "_Network", m_Menu_Network) );
  items().push_back( Gtk::Menu_Helpers::StockMenuElem( Gtk::Stock::HELP, m_Menu_Help) );
  
  // Parent pointer initialization
  m_parent = 0;
}


CMainMenuBar::~CMainMenuBar()
{

}


void CMainMenuBar::setParent(CMainWindow* parent)
{
  m_parent =  parent;
}


void CMainMenuBar::on_menu_file_new()
{
  
}


void CMainMenuBar::on_menu_file_open()
{
  m_parent->m_NetworksArea_Main.openCircuitFile();
}


void CMainMenuBar::on_menu_file_quit()
{

}


void CMainMenuBar::on_menu_edit_preferences()
{

}


void CMainMenuBar::on_menu_network_solve()
{
  m_parent->m_NetworksArea_Main.solveCircuit();
}


void CMainMenuBar::on_menu_help_about()
{
  RLC::CAboutDialog dialog;
  dialog.run();
}

