/*
 * RolLerCoaster Linear Networks Solver
 *
 * RolLerCoaster is the legal property of its developers, whose names are 
 * listed in the COPYRIGHT file distributed with this source distribution.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the free software foundation; either version 2 of the license, or
 * (at your option) any later version.
 *
 * this program is distributed in the hope that it will be useful,
 * but without any warranty; without even the implied warranty of
 * merchantability or fitness for a particular purpose.  see the
 * gnu general public license for more details.
 *
 * you should have received a copy of the gnu general public license
 * along with this program; if not, write to the free software
 * foundation, inc., 59 temple place, suite 330, boston, ma  02111-1307  usa
 *
 */
 
#ifndef MAINMENUBAR_HH
#define MAINMENUBAR_HH

#include <gtkmm/menubar.h>
#include <gtkmm/menu.h>

/**
 * Main RolLerCoaster namespace.
 * Namespace that consists of every class that is made for R(ol)L(er)C(oaster).
 */
namespace RLC
{
  class CMainWindow;

  /**
   * A upper menu class.
   */
  class CMainMenuBar : public Gtk::MenuBar
  {
    public:
      CMainMenuBar();
      ~CMainMenuBar();
      /**
       * A method used for setting a parent CMainWindow class pointer.
       * This is so that for this class we could get "upper" and use
       * public methods of others widgets in a window.
       * @param parent pointer to parent window
       * @see m_parent
       */
      void setParent(RLC::CMainWindow* parent);

      
    protected:
      // Menu elements signal handlers
      virtual void on_menu_file_new();
      virtual void on_menu_file_open();
      virtual void on_menu_file_quit();
      virtual void on_menu_edit_preferences();
      virtual void on_menu_network_solve();
      virtual void on_menu_help_about();
      
      // Widget for each menu (or submenu)
      Gtk::Menu m_Menu_File, m_Menu_Edit, m_Menu_Network, m_Menu_Help;

      /**
       * A pointer to a CMainWindow object that this object is inside.
       * @see setParent
       */
      RLC::CMainWindow* m_parent;

  };
  
}


#endif // MAINMENUBAR_HH

