/*
 * RolLerCoaster Linear Networks Solver
 *
 * RolLerCoaster is the legal property of its developers, whose names are 
 * listed in the COPYRIGHT file distributed with this source distribution.
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the free software foundation; either version 2 of the license, or
 * (at your option) any later version.
 *
 * this program is distributed in the hope that it will be useful,
 * but without any warranty; without even the implied warranty of
 * merchantability or fitness for a particular purpose.  see the
 * gnu general public license for more details.
 *
 * you should have received a copy of the gnu general public license
 * along with this program; if not, write to the free software
 * foundation, inc., 59 temple place, suite 330, boston, ma  02111-1307  usa
 *
 */
 
#ifndef MAINWINDOW_HH
#define MAINWINDOW_HH

#include <gtkmm/window.h>
#include <gtkmm/box.h>

#include "mainmenubar.hh"
#include "maintoolbar.hh"
#include "networksarea.hh"
#include "mainstatusbar.hh"

/**
 * Main RolLerCoaster namespace.
 * Namespace that consists of every class that is made for R(ol)L(er)C(oaster).
 */
namespace RLC
{

  /**
   * Main window class.
   * This class consists of other widget based class that are shown in GUI in
   * main editing window.
   */
  class CMainWindow : public Gtk::Window
  {
    friend class CMainToolbar;
    friend class CMainMenuBar;
    
    public:

      /**
       * Class constructor.
       * Read configuration, initiates main window elements and shows them.
       */ 
      CMainWindow();
      
      /**
       * Class destructor.
       * Cleans up pointer mess if there is any.
       */
      ~CMainWindow();
      
    protected:
      Gtk::VBox m_VBox_Main;
      RLC::CMainMenuBar m_MainMenuBar_Upper;
      RLC::CMainToolbar m_MainToolbar_Upper;
      RLC::CNetworksArea m_NetworksArea_Main;
      RLC::CMainStatusbar m_MainStatusbar_Bottom;
  };
  
}


#endif // MAINWINDOW_HH

