#include <signalframe.hh>
#include <iostream>
#include <iomanip>
#include <cmath>
#include <networksarea.hh>

using namespace RLC;

CSignalFrame::CSignalFrame()
{
  m_signal = 0;
  m_coords = Gdk::Point(0,0);
  
  // @FUTURE_CONFIG@ or make this default m_squareSize dependent;
  m_size   = Gdk::Point(300,200); 
  resetChartCoords();

  m_parent = 0;
}


// CSignalFrame::~CSignalFrame()
// {
// 
// }


void CSignalFrame::setCoords(Gdk::Point coords) 
{
  int squareSize = m_parent->m_SquareSize;
  int sizeY = m_size.get_y();
  // Reseting frame vertical position to visible if a frame is out
  // of a circuit drawing window.
  m_coords = Gdk::Point(coords.get_x(),
                        abs(coords.get_y()-sizeY) + sizeY);

  resetChartCoords();  

  std::cerr << coords.get_y()-sizeY << "+" << sizeY
            << "[" << coords.get_x() << "," << coords.get_y() << "]" << std::endl;
	    
}




void CSignalFrame::draw()
{
  Glib::RefPtr<Gdk::Pixmap> refPixmap_NetworksArea = m_parent->m_RefPixmap_NetworksArea;
  Glib::RefPtr<Gdk::GC> refGC_NetworksArea = m_parent->m_RefGC_NetworksArea;
  Glib::RefPtr<Pango::Context> refContext_Main = m_parent->m_RefContext_Main;
  int squareSize = m_parent->m_SquareSize;
  int nodeSize   = m_parent->m_NodeSize;
  

  // FRAME BACKGROUND DRAWING

  // TODO: think about changing colors in functions. Do this here or outside, before
  // function call??
  refGC_NetworksArea->set_foreground(m_parent->m_Color_FrameBG);
  
  refPixmap_NetworksArea->draw_rectangle(
      refGC_NetworksArea,
      1,
      m_coords.get_x(),
      m_coords.get_y()- m_size.get_y(),
      m_size.get_x(),
      m_size.get_y());

  
  refGC_NetworksArea->set_foreground(m_parent->m_Color_BG);

  // CHART FRAME DRAWING
 
  // @FUTURE_CONFIG@
  int marginWidth  = 5;
  int marginHeight = 5;
  
  Gdk::Point m_chartSize(m_size.get_x()-(m_chartCoords.get_x()-m_coords.get_x()+2*marginWidth),
                         m_size.get_y()-(-m_chartCoords.get_y()+m_coords.get_y()+6*marginHeight)); 
    
  refPixmap_NetworksArea->draw_rectangle(
      refGC_NetworksArea,
      0,
      m_chartCoords.get_x(),
      m_chartCoords.get_y()-m_chartSize.get_y(),
      m_chartSize.get_x(),
      m_chartSize.get_y());

  // CHART DESCRIPTIONS DRAWING
  
  Glib::RefPtr<Pango::Layout> text = Pango::Layout::create(refContext_Main);
  
  text->set_text(m_signal->getUnit());
  
  refPixmap_NetworksArea->draw_layout(
      refGC_NetworksArea,
      static_cast<int>( (m_coords.get_x()+marginWidth) ) ,
      static_cast<int>( (m_chartCoords.get_y()-m_chartSize.get_y()/2) ),
      text);

  
  text->set_text("s");
  
  refPixmap_NetworksArea->draw_layout(
      refGC_NetworksArea,
      static_cast<int>( (m_chartCoords.get_x()+m_coords.get_x()+m_size.get_x())/2 ),
      static_cast<int>( (m_chartCoords.get_y()+m_coords.get_y())/2 ),
      text);

  text->set_text(m_signal->getText());
  
  refPixmap_NetworksArea->draw_layout(
      refGC_NetworksArea,
      static_cast<int>( (2*m_coords.get_x()+m_size.get_x())/2 ),
      static_cast<int>( (m_coords.get_y()-m_size.get_y()+marginHeight) ),
      text);
  

  // CHART VALUE DESCRIPTIONS DRAWING
  
  double maxTime = 2.0;
  double timeStep = maxTime/m_chartSize.get_x();
  double maxValue = m_signal->time(0);
  double minValue = maxValue;
  double value;
  for (double t = 0; t < maxTime; t+=timeStep) {
    value = m_signal->time(t);
    if (maxValue < value)
      maxValue = value;
    else
      if (minValue > value)
	minValue = value;
    
  }

  int xBetweenNums = 3;
  int yBetweenNums = 3;

  int pinWidth     = m_chartSize.get_x()/50;
  double yValueStep = (maxValue-minValue)/(yBetweenNums+1);
  int yCoordsStep = m_chartSize.get_y()/(yBetweenNums+1);
  double xValueStep = maxTime/(yBetweenNums+1);
  int xCoordsStep = m_chartSize.get_x()/(yBetweenNums+1);


  
  {
  int y_ = m_chartCoords.get_y();

  for (int i=0; i <= yBetweenNums; ++i) {

    refPixmap_NetworksArea->draw_line(
	refGC_NetworksArea,
	m_chartCoords.get_x(),
	y_,
	m_chartCoords.get_x()+pinWidth,
	y_);

    refPixmap_NetworksArea->draw_line(
	refGC_NetworksArea,
	m_chartCoords.get_x()+m_chartSize.get_x(),
	y_,
	m_chartCoords.get_x()+m_chartSize.get_x()-pinWidth,
	y_);

    {
      std::ostringstream vos;
      vos << std::setprecision(2) << fixed << setw(3) << minValue+yValueStep*i; 

      text->set_text(vos.str());

      refPixmap_NetworksArea->draw_layout(
	  refGC_NetworksArea,
	  static_cast<int>( (m_chartCoords.get_x()+m_coords.get_x())/2-2*marginWidth),
	  static_cast<int>( (y_-2*marginHeight) ),
	  text);

    }

    y_-=yCoordsStep;
  }

  }

  {
    std::ostringstream vos;
    vos << std::setprecision(2) << fixed <<maxValue; 

    text->set_text(vos.str());

    refPixmap_NetworksArea->draw_layout(
	refGC_NetworksArea,
	static_cast<int>( (m_chartCoords.get_x()+m_coords.get_x())/2-2*marginWidth),
	static_cast<int>( (m_chartCoords.get_y()-m_chartSize.get_y()-2*marginHeight) ),
	text);

  }

  // {
    // std::ostringstream vos;
    // vos << std::setprecision(2) << minValue; 
// 
    // text->set_text(vos.str());
// 
    // refPixmap_NetworksArea->draw_layout(
	// refGC_NetworksArea,
	// static_cast<int>( (m_chartCoords.get_x()+m_coords.get_x())/2 ),
	// static_cast<int>( m_chartCoords.get_y() ),
	// text);
// 
  // }

  {
  int x_ = m_chartCoords.get_x();

  for (int i=0; i <= xBetweenNums; ++i) {

    refPixmap_NetworksArea->draw_line(
	refGC_NetworksArea,
	x_,
	m_chartCoords.get_y(),
	x_,
	m_chartCoords.get_y()-pinWidth);

    refPixmap_NetworksArea->draw_line(
	refGC_NetworksArea,
	x_,
	m_chartCoords.get_y()-m_chartSize.get_y(),
	x_,
	m_chartCoords.get_y()-m_chartSize.get_y()+pinWidth);

    {
      std::ostringstream vos;
      vos << std::setprecision(2) << fixed << setw(3) << xValueStep*i; 

      text->set_text(vos.str());

      refPixmap_NetworksArea->draw_layout(
	  refGC_NetworksArea,
	  static_cast<int>( (x_-3*marginWidth) ),
	  static_cast<int>( (m_chartCoords.get_y()+2*marginHeight) ),
	  text);

    }

    x_+=xCoordsStep;
  }

  }
  {
    std::ostringstream vos;
    vos << std::setprecision(2) << fixed << setw(3) << maxTime; 

    text->set_text(vos.str());

    refPixmap_NetworksArea->draw_layout(
	refGC_NetworksArea,
	static_cast<int>( m_chartCoords.get_x()+m_chartSize.get_x()-5*marginWidth),
	static_cast<int>( m_chartCoords.get_y()+2*marginHeight),
	text);

  }
  
  // DRAWING A CHART - AT LAST :)
  
  refGC_NetworksArea->set_foreground(m_parent->m_Color_Branch);

  double chartValueRatio = m_chartSize.get_y()/(maxValue-minValue);
  
  // previous point y coord (value), initialize by the first value (in first timeStep).
  // Here x=1.
  int y_ = yC(static_cast<int>( chartValueRatio*(-minValue+m_signal->time(timeStep)) ));
  int y;
  
  for (int x = 2; x < m_chartSize.get_x(); ++x) {
    y = yC(static_cast<int>( chartValueRatio*(-minValue+m_signal->time(x*timeStep)) ));
    refPixmap_NetworksArea->draw_line(
      refGC_NetworksArea,
      xC(x-1),
      y_,
      xC(x),
      y);
    
    y_ = y; 
  }

  // for (int x = 1; x < m_chartSize.get_x(); ++x)
  // refPixmap_NetworksArea->draw_point(
      // refGC_NetworksArea,
      // xC(x),
      // yC(static_cast<int>( chartValueRatio*(-minValue+m_signal->time(x*timeStep)) )));

  
  refGC_NetworksArea->set_foreground(m_parent->m_Color_FG);
  

  std::cerr << "CSignalFrame::draw() called [" << &(*m_signal) << "," 
            << m_signal->getDescription() << "," 
	    << m_signal->getText() << ","
	    << m_signal->getUnit() << ","
	    << m_signal->getEq() << "]" << std::endl;
  
}


int CSignalFrame::xF(int x)
{
  return m_coords.get_x()+x;
}


int CSignalFrame::yF(int y)
{
  return m_coords.get_y()-y;
}


int CSignalFrame::xC(int x)
{
  return m_chartCoords.get_x()+x;
}


int CSignalFrame::yC(int y)
{
  return m_chartCoords.get_y()-y;
}


void CSignalFrame::resetChartCoords() 
{
  m_chartCoords = Gdk::Point( static_cast<int>( m_coords.get_x()+0.2*m_size.get_x() ),
                              static_cast<int>( m_coords.get_y()-0.2*m_size.get_y() ));
}

